//!
//!

use super::models;
use crate::repositories;
use time::{Duration, OffsetDateTime};

pub struct Composition {
  jwt_secret: String,
  site_repository: repositories::site::repository::Repository,
}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Composition {
  ///
  pub fn new(jwt_secret: String) -> Composition {
    Composition {
      jwt_secret,
      site_repository: repositories::site::repository::Repository::new(),
    }
  }

  ///
  pub fn get_token(
    &self,
    issuer: String,
    session_id: String,
  ) -> Result<String, jsonwebtoken::errors::Error> {
    use jsonwebtoken::{encode, Algorithm, EncodingKey, Header};

    let header = Header::new(Algorithm::HS512);

    let issued_at = OffsetDateTime::now_utc();
    let expiration_at = issued_at + Duration::days(1);
    let claims = models::Claims {
      iss: issuer,
      iat: issued_at.unix_timestamp(),
      exp: expiration_at.unix_timestamp(),
      session_id,
    };

    let token = encode(
      &header,
      &claims,
      &EncodingKey::from_secret(self.jwt_secret.as_bytes()),
    )?;

    Ok(token)
  }
}
